package com.thisiscool.savethatcalendar;

import java.awt.*;
import java.awt.event.*;

import java.util.Calendar;
import java.util.GregorianCalendar;

// This program is freeware. I retain my copyright, but you can freely copy
// and distribute this. Try to give me credit if I deserve it.

/**
 * Title:        Save That Calendar!
 * Description:  Allows you to determine the next year in which a calendar for a given year can be reused.
 * Copyright:    Copyright (c) 2001
 * Company:      Mohan Embar, Inc.
 * @author Mohan Embar
 * @version 1.0
 */

public class SaveThatCalendarGUI
{
public SaveThatCalendarGUI(Container cnt)
{
    m_cnt = cnt;
    cnt.setFont(new Font("Helvetica", Font.PLAIN, 12));

    init();
        // initialize myself

    layoutUI();
        // lay out our UI components

    updateUI();
}

// private //
/**
 * My container
 */
private Container m_cnt;

/**
 * The input text field.
 */
private TextField m_txfInput;

/**
 * Our output year label.
 */
private List m_lstMatches;

/**
 * Our status label.
 */
private Label m_lblStatus;

private void init()
{
}

private void layoutUI()
{
    m_cnt.setLayout(new BorderLayout());

    Panel pnlMain = new Panel();
    pnlMain.setLayout(new BorderLayout());
    m_cnt.add(pnlMain, BorderLayout.CENTER);

    Panel pnlStatus = new Panel();
    m_cnt.add(pnlStatus, BorderLayout.SOUTH);
    pnlStatus.setLayout(new FlowLayout(FlowLayout.LEFT));
    pnlStatus.add(m_lblStatus = new Label("Enter the calendar year, then press [Find Matches]"));

    Panel pnlTop = new Panel();
    pnlTop.setLayout(new BorderLayout());
    pnlMain.add(pnlTop, BorderLayout.CENTER);

    Panel pnlLabels = new Panel();
    pnlLabels.setLayout(new BorderLayout());
    pnlTop.add(pnlLabels, BorderLayout.WEST);

    Panel pnlFields = new Panel();
    pnlFields.setLayout(new BorderLayout());
    pnlTop.add(pnlFields, BorderLayout.CENTER);

    pnlLabels.add(new Label("Current Year:"), BorderLayout.NORTH);
    pnlLabels.add(new Label("Matches:"), BorderLayout.CENTER);

    pnlFields.add(m_txfInput = new TextField(4), BorderLayout.NORTH);
    pnlFields.add(m_lstMatches = new List(5), BorderLayout.CENTER);

    Panel pnlButtons = new Panel();
    pnlMain.add(pnlButtons, BorderLayout.SOUTH);

    Button bnNextYear = new Button("Find Matches");
    pnlButtons.add(bnNextYear);
    bnNextYear.addActionListener
    (
        new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                onBnNextYear();
            }
        }
    );

    Frame frm;
    if (m_cnt instanceof Frame)
    {
        layoutFrame(pnlButtons);
        // java.awt.Frame-specific stuff
    }
}

private void layoutFrame(Panel pnlButtons)
{
    final Frame frm = (Frame) m_cnt;

    Button bnExit = new Button("Exit");
    pnlButtons.add(bnExit);
    bnExit.addActionListener
    (
        new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                onFileExit();
            }
        }
    );

    // If we close my frame, exit the app.
    frm.addWindowListener
    (
        new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                onFileExit();
            }
        }
    );

    frm.pack();
}

private void updateUI()
{
}

// Command handlers
private void onBnNextYear()
{
    try
    {
        String strYear = m_txfInput.getText();
        int nYear = Integer.parseInt(strYear);
        if (nYear<1800 || nYear>2500)
            throw new RuntimeException("Year out of range.");

        // Compute all matches
        int nSelIndex = -1;
        m_lstMatches.removeAll();
        GregorianCalendar gcalThis = new GregorianCalendar(nYear,0,1);
        for (int nCurYear=1800; nCurYear<=2200; ++nCurYear)
        {
            GregorianCalendar gcalCur =
                new GregorianCalendar(nCurYear,0,1);
            boolean bMatches =
                gcalThis.isLeapYear(nYear) == gcalCur.isLeapYear(nCurYear)
                &&
                gcalThis.get(Calendar.DAY_OF_WEEK) == gcalCur.get(Calendar.DAY_OF_WEEK);
            if (bMatches)
            {
                if (nCurYear == nYear)
                    nSelIndex = m_lstMatches.getItemCount();
                m_lstMatches.add(Integer.toString(nCurYear));
            }
        }
        m_lstMatches.select(nSelIndex);
        m_lstMatches.requestFocus();
    }
    catch (Throwable thw)
    {
        String strError = thw.getClass().getName()+": "+thw.getMessage();
        m_lblStatus.setText(strError);
    }
}

private void onFileExit()
{
    System.exit(0);
}
}
